c-----------------------------------------------------------------------
      program datsav
c-----------------------------------------------------------------------
c 
c --- DATSAV    Version: 1.1      Level: 070329                    MAIN
c
c     Copyright (c) 1999-2007 by Exponent, Inc.
c
c --- Written by Al Klausmann, Joe Scire and S. Du
c
c --- PURPOSE:
c                This program reads the Abbreviated, Space-Delimited
c                file created by NCDC from the TD9956 (DATSAV3) surface
c                meteorological data, and creates either a CALMET
c                SURF.DAT, HUSWO, or CD144 file for a time period
c                specified by the user.
c                This version is currently configured to create
c                ONLY the CD144 file.
c
c --- Updates:
c
c --- Modified: v1.0 020307 to v1.1 070329, D. Strimaitis
c     (1) For CD144, map WD=990 or ***  to '99', not '  ' (missing).
c         This will be interpreted as a missing WD in SMERGE unless WS=00
c         (calm).  The calm result from SMERGE will replace WD=99 with WD=0
c         in the output SURF.DAT file.
c         Original abbreviated DATSAV3 files used WD=000 for calms, but
c         recent files that do not adhere to this convention may use WD=990
c         or *** instead to indicate that the direction is uncertain during
c         calm periods.
c         NOTE:  WD=99 is not part of the CD144 format definition but has
c                been used in the past.
c     (2) Add command-line specification of control file (COMLINE)
c
c --- Modified: 2001-07-30 to 2002-03-07, D. Strimaitis
c     For CD144, map CLG=722 or 689 to unlimited as '---', not '   '
c     Modify WETBULB subroutine:
c     - Dry bulb and dew point temperature, and station pressure input
c     - Wet bulb temperature returned
c     - Vapor pressure from Goff-Gratch eqns.
c
c --- Modified: 2001-05-08 to 2001-07-30, D. Strimaitis
c     CD144 output format enforced (option removed from control file);
c     add time convention used in input data file (UTC or LST);
c     station ID requested from user (extract 1 station only);
c     Y2K compliant;
c     CD144 wind speed (kt) is computed as a Nearest Integer:
c     data type matched to format on internal writes
c
c --- Modified: 2000-12-05 to 2001-05-08, Benjamin de Foy
c     changes to ceiling height processing, !!check surf.dat output!!
c     (acg used to be 2 characters not 3 + unlimited was not dealt with)
c
c --- Modified: 2000-10-30 to 2000-12-05, David Healy (search on 'dsh')
c     visibility format changed in latest data from NCDC
c
c --- Modified: 2000-10-20 to 2000-10-30, Benjamin de Foy
c     write units and source in header
c     slight change to handle 3 line header better
c
c --- Modified: 2000-09-07 to 2000-10-20, Benjamin de Foy
c     convert weather codes for CD144 format (not for HUSWO & surf.dat)
c     comment out call to subroutine pcpcode - needs replacing
c
c --- Modified: 2000-01-28 to 2000-09-07, Benjamin de Foy
c     Read station elevation from input file
c     Minimum RH set to 1.0 to prevent crash in wetbulb, warning msg to lst
c     Fill in fields with zeroes for CD144 output
c
c --- Modified: 991014 to 2000-01-28, Benjamin de Foy
c     Sea Level pressure for CD144 corrected (it is not in in Hg)
c
c --- Modified: level 990111 to 991014, Benjamin de Foy
c     Convert sea level pressure to surface pressure if missing
c     Calculate Wet Bulb Temperature for CD144
c     Option added in input file to convert to HUSWO or CD144
c
c     DATSAV3 format (06/22/99) (Was DATSAV2: 10/04/96)
c       Differences: Ceiling height went from 2 digit code to
c       3 digit height in '00s feet. Format for Present Weather changed.
c       Station type indicator was eliminated
c     Bugs removed when going over new years and new months.
c     6 digit station ID's reduced to 5 digits (divided by 10)
c
c     Note: if program hangs up, it may be due to an unorthodox carriage return
c           in the DATSAV file. Perl script ascii2dos.pl solves the problem 
c           as does unix2dos.exe
c
c=======================================================================
c --- Commented by Shuming Du:
c --- The mapping of opaque cloud cover from TD9950 to SURF.DAT is not perfect:
c --- in TD9950 the cloud cover is for total cloud. Occasionally more info 
c --- about low, middle and high cloud type is available. This may be of help
c --- in determining whether the cloud is opaque. However, this info is RARELY 
c --- available, so it is assumed that TOTAL cloud cover can be used to 
c --- substitute OPAQUE cloud cover.
c --- The precipation data are not useful because amount of precipation is 
c --- reported every 6 hours.
c --- Also, the ipcode may be different for other TD9950 files. 
c --- At this time, I have only one file available in which mannual 
c --- station code was used. For different precipitation code, you might 
c --- need to modify the present program.
c --- End of SD comments
c=======================================================================

      parameter(io5=5,io6=16,io7=7,io8=8)
      integer yr,mo,dy,hr,mi
      integer istnid, isttmp, istn0
      Character*30 ihead
      character*132 cfname
      Character*80 filelst,filein,fileout
      Character*1 al,am,ah,aw,headchar,unused
      Character*3 acg
      character*2 aww(4),atsd
      Character*3 adir,aspd,agus,askc,avsb
      Character*3 atmp,adew,amax,amin
      character*3 autc
      Character*4 acall
      Character*5 aalt
      Character*6 aslp,astp,apcp6,apcp12,apcp24
      character*8 ver, level
      integer irh
      logical lfatal, lfirst
c HUSWO variables:
      integer yrh,moh,dyh,hrh,cehth,iwdrh,tskcovh,oskcovh
      integer ly1typ2h,ly1hth,ly2typ2h,ly2hth,ly3typ2h,ly3hth
      integer irhh,ipresr,iprec,snow
      real rh,spdh,tth,tdh,hvis
      character*1 asosflg,ttfl,presrfl,precfl
      character*8 pwth
      logical istflag
      character*40 coment
c CD144 variables:
      character*1 atsk,aosk
      character*2 awdr,asp
      character*3 ceht,avis,atd,att,atw,arh
      character*4 apresrl,apresr
      character*8 pwthcd

      real mbtoinhg  ! conversion factor for millibars to inches of Hg
      parameter (mbtoinhg = 0.02953)

      integer nelev ! No. of station pres. extrapolated from sea level p.
      real elevation ! elevation (in meters)

      iend=0
      nelev = 0     ! initialise count
c HUSWO initialisations:
      asosflg = ' '
      grad = 9999
      drad = 9999
      pwth = '99999999'
      ttfl = ' '
      presrfl = ' '
      iprec = 999
      precfl = ' '
      snow = 999
      istflag = .false.

c variables that are read but are not used:
c (this list is included to eliminate compiler warnings)
      unused = acall
      unused = agus
      unused = al
      unused = am
      unused = ah
      unused = aw
      unused = aalt
      unused = amax
      unused = amin
      unused = apcp6
      unused = apcp12
      unused = apcp24
      unused = atsd
c 'use' unused:
      headchar = unused

c --- Read control file
c ---------------------

C --- OPEN CONTROL FILE
      cfname='datsav.inp'
      call COMLINE(cfname)
      open(io5,file=cfname,status='OLD')

c --- Open the list file
      Read(io5,901)ihead,filelst
      Open(io6,file=filelst,form='formatted')

C --- WRITE HEADER TO LIST FILE
      ver = '1.1'
      level = '070329'
      write(io6,5) ver,level
5     format(///,26x,'DATSAV OUTPUT SUMMARY',/,19x,'VERSION:  ',A8,
     1       ' LEVEL:  ',A8///)
      Write(io6,901)ihead,filelst

c --- Read remaining control file inputs
      read(io5,'(a30,i4)') ihead,jyrb
      write(io6,'(a30,i4)') ihead,jyrb
      read(io5,'(a30,i3)') ihead,julb
      write(io6,'(a30,i3)') ihead,julb
      read(io5,'(a30,i2)') ihead,jhrb
      write(io6,'(a30,i2)') ihead,jhrb
      read(io5,'(a30,i4)') ihead,jyre
      write(io6,'(a30,i4)') ihead,jyre
      read(io5,'(a30,i3)') ihead,jule
      write(io6,'(a30,i3)') ihead,jule
      read(io5,'(a30,i2)') ihead,jhre
      write(io6,'(a30,i2)') ihead,jhre
      read(io5,'(a30,i6)') ihead,istn0
      write(io6,'(a30,i6)') ihead,istn0
      read(io5,'(a30,i3)') ihead,ibtz
      write(io6,'(a30,i3)') ihead,ibtz
      read(io5,'(a30,a3)') ihead,autc
      write(io6,'(a30,a3)') ihead,autc

c --- open the input file (DATSAV)
      Read(io5,901)ihead,filein
      Write(io6,901)ihead,filein
      Open(io7,file=filein,form='formatted',status='OLD')

c --- open the output file 
      Read(io5,901)ihead,fileout
      Write(io6,901)ihead,fileout
      Open(io8,file=fileout,form='formatted')

c --- Set tolerance (minutes) for assigning hour
      read(io5,906) ihead,minpre
      write(io6,906) ihead,minpre
      read(io5,906) ihead,mincur
      write(io6,906) ihead,mincur

c --- Enforce CD144 format for output file
c bdf : read opflag: 1=surf.dat, 2=HUSWO, 3=CD144
c ---  read(io5,906) ihead,opflag
c ---  write(io6,906) ihead,opflag
      opflag=3

c bdf : read station elevation
      read(io5,907) ihead,elevation
      write(io6,907) ihead,elevation

      write (*,800) filein,fileout
 800  format(' Input and Output Files: ',A12,', ',A12)


c --- Process information
c -----------------------
      lfatal=.FALSE.

c --- Set number of hours to ADD to INPUT file times to get LST
c --- (typically converting from UTC to LST)
      if(autc.EQ.'UTC' .OR. autc.EQ.'utc') then
         ihradj=-ibtz
      elseif(autc.EQ.'GMT' .OR. autc.EQ.'gmt') then
         ihradj=-ibtz
      elseif(autc.EQ.'LST' .OR. autc.EQ.'lst') then
         ihradj=0
      else
         lfatal=.TRUE.
         write(*,*)
         write(*,*)'FATAL ERROR: time in data file is UTC or LST'
         write(*,*)'             User input not recognized: ',autc
      endif

c --- Special Y2K QA on starting year
      call QAYR4(io6,jyrb,0,ierr)
      if(ierr.NE.0) then
         lfatal=.TRUE.
         write(io6,*) 'Starting Year Invalid'
      endif

c --- Make sure ending year is YYYY (Y2K)
      call YR4(io6,jyre,ierr)
      if(ierr.NE.0) then
         lfatal=.TRUE.
         write(io6,*) 'Ending Year Invalid'
      endif

c --- Check for fatal errors
      if(lfatal) stop 'Abnornal Termination!'

c --- Account for hour=24 entry (should not happen, but ...)
      if(jhrb.EQ.24) then
         jhrb=23
         nhr=1
         call INCR(io6,jyrb,julb,jhrb,nhr)
      endif
      if(jhre.EQ.24) then
         jhre=23
         nhr=1
         call INCR(io6,jyre,jule,jhre,nhr)
      endif

c --- Set date-time for requested period (LST)
      jtimeb=jyrb*100000+julb*100+jhrb
      jtimee=jyre*100000+jule*100+jhre
      write (*,*) 'Start and End Time:',jtimeb,jtimee

c --- Write out header info.
c --- Set number of stations as 1
      nsta = 1
      if (opflag.eq.1) then
         write(io8,908)jyrb,julb,jhrb,jyre,jule,jhre,ibtz,nsta
      elseif (opflag.eq.2) then
         write (coment,920) filein
 920     format('units="English",source="',A12,'"')
c         write(io8,911) 1,2,5,6,7,11,12,14,16,17,18
         write(io8,913) 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
     &        16,17,18,19,20,coment
      endif

 901  Format(a30,a80)
c old (10/04/96) format:
c 902  format(i6,1x,a4,1x,i4,4i2,3(1x,a3),1x,a2,1x,a3,
c     1 3(1x,a1),4(1x,a3),1x,a2,2(1x,a1),2(1x,a3),1x,a6,
c     1 1x,a5,1x,a6,2(1x,a3),1x,a5,2(1x,a6),1x,a3)
c bdf: new (990825) format:
c            stn   call  date   dir spd gus clg   skc
c902  format(i6,1x,a4,1x,i4,4i2,3(1x,a3),1x,a3,1x,a3,
c      l m h       vsb ww ww ww ww w  temp dewp  slp
c    1 3(1x,a1),1x,a3,4(1x,a2),1x, a1,2(1x,a4),1x,a6,
c         alt   stp max min    pcp6 pcp12 pcp24 sd
c    1 1x,a5,1x,a6,2(1x,a3),1x,a5,  2(1x,a6),1x,a3)
c dsh: new (991205) format:
c            stn   call  date   dir spd gus clg   skc
 902  format(i6,1x,a4,1x,i4,4i2,3(1x,a3),1x,a3,1x,a3,
c      l m h       vsb ww ww ww ww w  temp dewp  slp
     1 3(1x,a1),1x,a4,4(1x,a2),1x, a1,2(1x,a4),1x,a6,
c         alt   stp max min    pcp6 pcp12 pcp24 sd
     1 1x,a5,1x,a6,2(1x,a3),1x,a5,  2(1x,a6),1x,a3)

 903  format(3i4)
 904  format((2f12.6,2i5,f12.6,i5,f12.6,i5))
 905  format(f12.6)
 906  Format(a30,i3)
 907  Format(a30,f12.3)
 908  format(2(i6,2i4),2i4)
c 909  format('Station ID:',i8) ! bdf: this causes windroses to bomb
 909  format(i8)

c Complete Huswo data record (NB: istnid = 5 digit, not 6)
c            istnid asosflg yrh       grad  drad  tsk osk   tth
 912  format(I5,    A1,     I4,3I2,1X,I4,1X,I4,1X,I2, I2,1X,F5.1,
c          ttfl  tdh     irhh  ipresr   iwdrh spdh    hvis    cehth pwth
     &     A1,1X,F5.1,1X,I3,1X,I4,A1,1X,I3,1X,F4.1,1X,F6.1,1X,I5,1X,A8,
c             lytyp,ht                   iprec precfl snow   
     &     1X,I2,I3,1X,I2,I3,1X,I2,I3,2X,I3,   A1,2X, I3)
c Huswo header format:
 913  format(T3,I1,T9,I1,T18,I1,T23,I1,T28,I1,T30,I1,T35,I1,T42,I1,
     &     T47,I1,T50,I2,T56,I2,T60,I2,T67,I2,T73,I2,T81,I2,
     &     T88,I2,T94,I2,T100,I2,T107,I2,T112,I2,2X,A40)

c CD144:
c            idstn,date,ceht, avis,pwth,apresrl,atd,awdr,asp,
 914  format(I5,   4I2.2, A3,4X, A3, A8,     A4, A3,  A2, A2,
c                apresr,att,atw,arh,tskcov,  oskcov
     &               A4, A3, A3, A3,    A1,22X,  A1)
c CD144: format for missing data:
 915  format(I5,   4I2.2)

c --- Set logical for 1-time operations
      lfirst=.TRUE.

c --- Initialize the date-time for the 'current hour'
      idathr=jtimeb

c --- Skip header
c --- Check if the DATSAV input file has a 3-line header. If yes, skip 
c --- the first 3 lines.
      read(io7,'(A1)') headchar
      backspace io7

      if(headchar.eq.'S'.or.headchar(1:1).eq.'s') then
c 1 line header:
         read(io7,*)
      else
C assume 3 line header:
         read(io7,*)
         read(io7,*)
         read(io7,*)
      endif

c --- Get data record
 100  read(io7,902,end=990) isttmp,acall,yr,mo,dy,hr,mi,adir,aspd,
     1  agus,acg,askc,al,am,ah,avsb,aww(1),aww(2),aww(3),aww(4),aw,
     1  atmp,adew,aslp,aalt,astp,amax,amin,apcp6,apcp12,apcp24,atsd

c --- Skip record if this is not the requested station ID
      if(istn0.NE.isttmp) goto 100

c end of data reached: fill in with missing code:
      if(yr.eq.0) go to 990

      if(lfirst) then
         if (opflag.eq.1) then
            write(io8,909)isttmp ! write out this to header only once
         endif
c ---    process station id code (5 digit version)
         if ((opflag.eq.2.or.opflag.eq.3).and.isttmp.gt.99999) then
            ileft=isttmp/100000
            istnid=isttmp-(ileft*100000)
            istflag = .true.
         else
            istnid = isttmp
         endif
         lfirst=.FALSE.
      endif    

c missing hour: hr=99
      if (hr.eq.99) goto 100

c --- Make sure year is YYYY (Y2K)
      call YR4(io6,yr,ierr)
      if(ierr.NE.0) then
         write(io6,*) 'Invalid Year in current data record'
         stop
      endif

c --- Compute the julian day
      call JULDAY(io6,yr,mo,dy,jul)

      if(ihradj.NE.0) then
c ---    Convert time in the input data file from UTC (GMT) to
c ---    Local Standard Time (LST)
         call INCR(io6,yr,jul,hr,ihradj)
         call GRDAY(io6,yr,jul,mo,dy)
      endif

      itime=yr*100000+jul*100+hr
      if(itime.gt.jtimee) go to 999

c --- Check if special observations can be used to substitute missing
c --- obs. hour
c --- Al K. suggests that we should use the last obs. of current hour 
c --- and 00 of next hour to fill in the missing data at the next hour. 2/1/99 SD
      if(mi.ge.minpre) then 
        iyr0 = yr
        idy0 = jul
        ihr0 = hr

c ---   Read in the next record
101     read(io7,902,end=990) isttmp,acall,yr,mo,dy,hr,mi,adir,aspd,
     1  agus,acg,askc,al,am,ah,avsb,aww(1),aww(2),aww(3),aww(4),aw,
     1  atmp,adew,aslp,aalt,astp,amax,amin,apcp6,apcp12,apcp24,atsd
c ---   Skip record if this is not the requested station ID
        if(istn0.NE.isttmp) goto 101
        if(yr.eq.0) go to 990
        call YR4(io6,yr,ierr)
        if(ierr.NE.0) then
           write(io6,*) 'Invalid Year in current data record'
           stop
        endif
        call JULDAY(io6,yr,mo,dy,jul)
        if(ihradj.NE.0) then
           call INCR(io6,yr,jul,hr,ihradj)
           call GRDAY(io6,yr,jul,mo,dy)
        endif
        backspace io7
        backspace io7
        call DELTT(iyr0,idy0,ihr0,yr,jul,hr,idiffhr)

        iuse = 0 ! flag used to fill missing data
        min = 1 ! flag used to fill missing data

        if(mi.ne.0.and.idiffhr.eq.1) then
          iuse =1 
          min = 0
        elseif(idiffhr.gt.1) then
          iuse =1 
          min = 0
        endif

102     read(io7,902,end=990) isttmp,acall,yr,mo,dy,hr,mi,adir,aspd,
     1  agus,acg,askc,al,am,ah,avsb,aww(1),aww(2),aww(3),aww(4),aw,
     1  atmp,adew,aslp,aalt,astp,amax,amin,apcp6,apcp12,apcp24,atsd
c ---   Skip record if this is not the requested station ID
        if(istn0.NE.isttmp) goto 102
        if(yr.eq.0) go to 990
        call YR4(io6,yr,ierr)
        if(ierr.NE.0) then
           write(io6,*) 'Invalid Year in current data record'
           stop
        endif
        call JULDAY(io6,yr,mo,dy,jul)
        nhr = ihradj + iuse
        if(nhr.NE.0) then
           call INCR(io6,yr,jul,hr,nhr)
           call GRDAY(io6,yr,jul,mo,dy)
        endif
        if(iuse.eq.1.and.min.eq.0) then
          mi = 0
        endif
      endif
      itime=yr*100000+jul*100+hr
      if(itime.gt.jtimee) go to 999
c --- End of handling special observation

c --- Process wind direction
 111  if((adir(1:1).eq.'*').or.(adir(2:2).eq.'*').
     1  or.(adir(3:3).eq.'*')) then
        wd = 9999.
        iwdrh = 999
c 070329   awdr = '  '
        awdr = '99'
      else      
        read(adir,*) wd
        iwdrh = wd
        write(awdr,'(I2.2)') NINT(wd/10.)
      endif
      if(wd.gt.360) then
         wd = 9999.
         iwdrh = 999
c 070329   awdr = '  '
        awdr = '99'
      endif

c --- Process wind speed
      if((aspd(1:1).eq.'*').or.(aspd(2:2).eq.'*').
     1  or.(aspd(3:3).eq.'*')) then
        ws = 9999.
        spdh = 99.9
        asp = '  '
      else      
        read(aspd(1:3),*) iws
c ---   Convert from mph to m/s for SURF.DAT
        ws = iws*1609./3600.
c ---   Represent integer mph as real mph for HUSWO
        spdh = iws
c       ispd = iws*0.86897624 ! convert from mph to knots
c ---   Use nearest integer after converting mph to knots for CD144
        ispd = NINT(spdh*0.86897624)
        if (ispd.lt.100) then
           write(asp,'(I2.2)') ispd
        else
           asp = '--'
        endif
      endif

c --- Process ceiling height
      if((acg(1:1).eq.'*').or.(acg(2:2).eq.'*'))then
        iceil = 9999
        cehth = 99999
        ceht = '   '
      else      
        read(acg,*) ichcode
c        call ceilingh(ichcode,ceiling,iyr,iday,ihr)
c Old Datsav2 format: ichcode = code for ceiling height in table (10/04/96)
c        if(ceiling.eq.9999) then
c          iceil = 9999
c        else
c          iceil = 0.5+ceiling*0.01/0.3048 ! convert from m to hundreds of feet
c        endif
c New Datsav2 format: ichcode = '00s of feet (6/22/99)
        if (ichcode.eq.689.or.ichcode.eq.722) then 
c unlimited ceiling:
           iceil = 9999
           cehth = 77777
           ceht = '---'
        else
           iceil = ichcode      ! '00s of feet for SURF.DAT
           cehth = ichcode*100  ! feet for HUSWO
           write(ceht,'(I3.3)') ichcode
        endif
        if (ichcode.eq.999) then
           ceht = '   '
        elseif(ichcode.gt.990) then
           ceht = '990'
        endif
      endif

c --- Process cloud cover
      if((askc(1:1).eq.'*').or.(askc(2:2).eq.'*')
     1  .or.(askc(3:3).eq.'*'))then
        icc = 9999
        tskcovh = 99
        atsk = ' '
      elseif(askc .eq. 'CLR') then
        icc = 0
        tskcovh = 0
        atsk = '0'
      elseif(askc .eq. 'SCT') then
        icc = 3    
        tskcovh = 3
        atsk = '3'
      elseif(askc .eq. 'BKN') then
        icc = 7    
        tskcovh = 7
        atsk = '7'
      elseif(askc .eq. 'OVC') then
        icc = 10   
        tskcovh = 10
        atsk = '-'
      elseif(askc .eq. 'OBS') then
        icc = 10    
        tskcovh = 10
        atsk = '-'
      elseif(askc .eq. 'POB') then
         icc = 10    
         tskcovh = 10
         atsk = '-'
c bdf: add else for undefined sky cover
      else
         icc = 9999
         tskcovh = 99
         atsk = ' '
      endif
c bdf: assume opaque = total (See PCRAMMET manual, pg4-9 (1 layer only)
      oskcovh = tskcovh
      aosk = atsk

c --- Process Cloud layers
c bdf: this should probably be omitted
      ly1typ2h = 99
      ly2typ2h = 99
      ly3typ2h = 99
      if (cehth.eq.99999) then
         ly1hth = 999
         ly2hth = 999
         ly3hth = 999
      else
         ly1hth = cehth/100
         ly2hth = cehth/100
         ly3hth = cehth/100
      endif

c --- Process visibility
      if (avsb(1:1).eq.'*'.or.avsb(2:2).eq.'*'.or.avsb(3:3).eq.'*')then
         hvis = 9999.9
         avis = '   '
      else
         read (avsb,*) hvis
         if (hvis.le.0.375) then
            write(avis,'(I3.3)') NINT(hvis*16.0)
         elseif (hvis.le.0.5) then
            write(avis,'(I3.3)') 7
         elseif (hvis.le.0.675) then
            write(avis,'(I3.3)') 8
         elseif (hvis.le.0.75) then
            write(avis,'(I3.3)') 9 
         elseif (hvis.le.1.0) then
            write(avis,'(I3.3)') 10
         elseif (hvis.le.1.125) then
            write(avis,'(I3.3)') 12
         elseif (hvis.le.1.25) then
            write(avis,'(I3.3)') 14
         elseif (hvis.le.1.375) then
            write(avis,'(I3.3)') 16
         elseif (hvis.le.1.5) then
            write(avis,'(I3.3)') 17
         elseif (hvis.le.1.625) then
            write(avis,'(I3.3)') 18
         elseif (hvis.le.1.75) then
            write(avis,'(I3.3)') 19
         elseif (hvis.le.2.00) then
            write(avis,'(I3.3)') 20
         elseif (hvis.le.2.25) then
            write(avis,'(I3.3)') 24
         elseif (hvis.le.2.5) then
            write(avis,'(I3.3)') 27
         elseif (hvis.le.3.0) then
            write(avis,'(I3.3)') 30
         elseif (hvis.le.15.0) then
            write(avis,'(I3.3)') NINT(hvis*10.0)
         elseif (hvis.le.99.0) then
            write(avis,'(I3.3)') NINT((hvis-mod(hvis,5.))*10.0)
         else
            avis = '990'
         endif
      endif

c --- Process temperature
      if(atmp(1:1).eq.'*'.or.atmp(2:2).eq.'*'.or.atmp(3:3).eq.'*') then
        tempk = 9999.
        tth = 999.9
        att = '   '
      else
        read(atmp,*) tempf
        tth = tempf
        if (tempf.lt.0) then
           write(att,'("-",I2.2)') NINT(-tempf)
        else
           write(att,'(I3.3)') NINT(tempf)
        endif
        tempk = 273.16+ (tempf-32.)*5./9.
      endif

c --- Process dew temperature
      if(adew(1:1).eq.'*'.or.adew(2:2).eq.'*'.or.adew(3:3).eq.'*') then
        tdewk = 9999.
        tdh = 999.9
        atd = '   '
      else
        read(adew,*) tdewf
        tdh = tdewf
        if (tdewf.lt.0) then
           write(atd,'("-",i2.2)') NINT(-tdewf)
        else
           write(atd,'(i3.3)') NINT(tdewf)
        endif
        tdewk = 273.16+ (tdewf-32.)*5./9.
      endif

c --- Now calculate relative humidity
      if((tempk.ne.9999.).and.(tdewk.ne.9999.)) then
         tnum = 173.-0.1*tempf+tdewf
         tdenom= 173.+0.9*tempf
         rh = (tnum/tdenom)**8
         irh = NINT(100.*rh)
         if (irh.gt.100) then
           irh = 100
           write(io6,700) iyr,mo,dy,hr,tempf,tdewf
 700       format('RH reset to 100.0 at date:',4I2.2,
     &          ', Tdry:',F6.2,' Tdew:',F6.2)
        elseif (irh.le.0) then
           irh = 1
           write(io6,701) iyr,mo,dy,hr,tempf,tdewf
 701       format('RH reset to 1.0 at date:',4I2.2,
     &          ', Tdry:',F6.2,' Tdew:',F6.2)
         endif
         write(arh,'(I3.3)') irh
         rh = irh
         irhh = irh
      else
         irh = 9999
         irhh = 999
         arh = '   '
         atw = '   '
      endif

c --- Process station pressure
      if(astp(1:1).eq.'*'.or.astp(2:2).eq.'*'.or.astp(3:3).eq.'*'
     1  .or.astp(4:4).eq.'*'.or.astp(5:5).eq.'*'
     1  .or.astp(6:6).eq.'*') then
        pres = 9999.
        ipresr = 9999
        apresr = '    '
      else
        read(astp,*) pres
c convert millibars to 1/100 inches of Mercury
        ipresr = pres * mbtoinhg * 100.0
        write(apresr,'(I4)') ipresr
      endif

c --- Process sea level pressure
      if(aslp(1:1).eq.'*'.or.aslp(2:2).eq.'*'.or.aslp(3:3).eq.'*'
     1  .or.aslp(4:4).eq.'*'.or.aslp(5:5).eq.'*'
     1  .or.aslp(6:6).eq.'*') then
        presl = 9999.
        ipresrl = 9999
        apresrl = '    '
      else
        read(aslp,*) presl
c convert to CD144 requirements for Sea Level Pressure
        if (presl.ge.1000.0) then
           ipresrl = nint((presl-1000)*10)
        else
           ipresrl = nint(presl*10)
        endif
        write(apresrl,'(I4.4)') ipresrl
      endif
c --- if station pressure is missing, approximate from sea level pressure
      if (ipresr.eq.9999.and.ipresrl.ne.9999.and.tempk.ne.9999.) then
         if (elevation.ne.-1.0) then
            nelev = nelev + 1   ! count number of updated pressures
            pres = presl * exp(-elevation/(29.3*tempk))
c convert millibars to 1/100 inches of Mercury
            ipresr = pres * mbtoinhg * 100.0
            write(apresr,'(I4)') ipresr
         else
            nelev = nelev - 1
            pres = 9999.0
            ipresr = 9999
            apresr = '    '
         endif
      endif

c --- Process wet-bulb temperature
      if((tempk.lt.9998.).and.(tdewk.lt.9998.)) then
         pmbsta = 1000.
         if(pres.LT.9998.) pmbsta=pres
         call wetbulb (tempf,tdewf,pmbsta,tw,rh2)
         if (tw.ne.999.9) then
            itw = NINT(tw)
            if (itw.lt.0) then
               write(atw,'("-",I2.2)') -itw
            else
               write(atw,'(I3.3)') itw
            endif
c --- REPLACE RH
c ---       irh2 = NINT(100.*rh2)
c ---       write(arh,'(I3.3)') irh2
c ---       rh = irh2
c ---       irhh = irh2
         else
c error calculating wet bulb temperature, write date info:
            write(*,*) 'Date:',yr,mo,dy,hr,mi
            atw = '   '
         endif
      else
         tw = 999.9
         atw = '   '
      endif


c --- Process for ipcode !bdf: this is out of date (aww1-3 are no longer read)
      ipcode = 9999
c      call pcpcode(aww1,aww2,aww3,ipcode,tempk)
c --- Process weather code for CD144
      call weather(pwthcd,aww)
c      if (pwthcd.ne.'        '.and.pwthcd.ne.'00000000') then
c         write (*,1348) iyr,mo,dy,ihr,pwthcd,
c     &        aww(1),aww(2),aww(3),aww(4)
 1348    format(' ',4i2,' CD144: ',A8,' d3l:',4(1x,a2))
c      endif

c --- Write to output file
c ------------------------
888   continue
      if(idathr.EQ.itime) then
c ---    Write out met info if obs. time is not beyond the cutoff time
         if(mi.le.mincur) then
            if(opflag.eq.1) then
c ---          SURF.DAT
               write(io8,903) yr,jul,hr
               write(io8,904) ws,wd,iceil,icc,tempk,irh,pres,ipcode
            elseif(opflag.eq.2) then
c ---          HUSWO
c ---          Set date-time for HUSWO (1-24 convention)
               yrh=yr
               julh=jul
               moh=mo
               dyh=dy
               hrh=hr
               if(hrh.EQ.0) then
                  call INCR(io6,yrh,julh,hrh,-1)
                  call GRDAY(io6,yrh,julh,moh,dyh)
                  hrh=24
               endif
               write(io8,912) istnid,asosflg,yrh,moh,dyh,hrh,grad,drad,
     &              tskcovh,oskcovh,tth,ttfl,tdh,irhh,ipresr,presrfl,
     &              iwdrh,spdh,hvis,cehth,pwth,
     &              ly1typ2h,ly1hth,ly2typ2h,ly2hth,ly3typ2h,ly3hth,
     &              iprec,precfl,snow
            elseif(opflag.eq.3) then
c ---          CD144
c ---          Set date-time for CD144 (YY convention for year)
               icent=yr/100
               iyr=yr-icent*100
               write(io8,914) istnid,iyr,mo,dy,hr,ceht,avis,pwthcd,
     &             apresrl,atd,awdr,asp,apresr,att,atw,arh,atsk,aosk
            endif
         else
c ---       Write out missing indicators
            if(opflag.eq.1) then
c ---          SURF.DAT
               write(io8,903) yr,jul,hr
               write(io8,904) 9999.,9999.,9999,9999,9999.,9999,
     1               9999.,9999
            elseif(opflag.eq.2) then
c ---          HUSWO
c ---          Set date-time for HUSWO (1-24 convention)
               yrh=yr
               julh=jul
               moh=mo
               dyh=dy
               hrh=hr
               if(hrh.EQ.0) then
                  call INCR(io6,yrh,julh,hrh,-1)
                  call GRDAY(io6,yrh,julh,moh,dyh)
                  hrh=24
               endif
               write(io8,912) istnid,asosflg,yrh,moh,dyh,hrh,9999,9999,
     &               99,99,999.9,' ',999.9,999,9999,' ',
     &               999,99.9,9999.9,99999,'99999999',
     &               99,999,99,999,99,999,
     &               999,' ',999
            elseif(opflag.eq.3) then
c ---          CD144
c ---          Set date-time for CD144 (YY convention for year)
               icent=yr/100
               iyr=yr-icent*100
               write(io8,915) istnid,iyr,mo,dy,hr
            endif
         endif
c ---    Increment date-time for next hour
         call INCR(io6,yr,jul,hr,1)
         idathr=yr*100000+jul*100+hr
      endif

      if(iend.eq.1) go to 444

c --- Fill in missing data if the current hour is not in the input file
      if(idathr.LT.itime) then
c ---    Extract current date-time variables
         call DEDAT(idathr,iyrc,ijulc,ihrc)
         call GRDAY(io6,iyrc,ijulc,imoc,idyc)
         write(io6,*) 'a: Missing data at',iyrc,ijulc,ihrc

         if(opflag.eq.1) then
c ---       SURF.DAT
            write(io8,903) iyrc,ijulc,ihrc
            write(io8,904) 9999.,9999.,9999,9999,9999.,9999,
     1          9999.,9999
         elseif (opflag.eq.2) then
c ---       HUSWO
c ---       Set date-time for HUSWO (1-24 convention)
            yrh=iyrc
            julh=ijulc
            moh=imoc
            dyh=idyc
            hrh=ihrc
            if(hrh.EQ.0) then
               call INCR(io6,yrh,julh,hrh,-1)
               call GRDAY(io6,yrh,julh,moh,dyh)
               hrh=24
            endif
            write(io8,912) istnid,asosflg,yrh,moh,dyh,hrh,9999,9999,
     &             99,99,999.9,' ',999.9,999,9999,' ',
     &             999,99.9,9999.9,99999,'99999999',
     &             99,999,99,999,99,999,
     &             999,' ',999
         elseif (opflag.eq.3) then
c ---       CD144
c ---       Set date-time for CD144 (YY convention for year)
            icent=iyrc/100
            iyr=iyrc-icent*100
            write(io8,915) istnid,iyr,imoc,idyc,ihrc
         endif
c ---    Increment date-time for next hour
         call INCR(io6,iyrc,ijulc,ihrc,1)
         idathr=iyrc*100000+ijulc*100+ihrc
c ---    Re-enter this section and test date-time again
         goto 888
      endif
      go to 100

 990  continue

c --- EOF reached in data file
c ----------------------------

      if(LFIRST) then
c ---    No data for selected station
         write(io6,*)'Selected station was not found:  ',istn0
         write(*,*)'Selected station was not found:  ',istn0
         goto 999
      endif

c --- See if the last record read for this station should be used here
      if(mi.ge.minpre .AND. istn0.EQ.isttmp) then
c ---    Increment hour of last data read that had been rejected
         call INCR(io6,yr,jul,hr,1)
         call GRDAY(io6,yr,jul,mo,dy)
         mi = 0
c ---    Set a flag that this record is the last one.
         iend = 1
         goto 111
      endif

 444  continue
c --- Fill missings to end of requested period
      if(jtimee.LT.idathr) then
         goto 999
      else
c ---    Extract current date-time variables
         call DEDAT(idathr,iyrc,ijulc,ihrc)
         call GRDAY(io6,iyrc,ijulc,imoc,idyc)
         write(io6,*) 'b: Missing data at',iyrc,ijulc,ihrc

         if(opflag.eq.1) then
c ---       SURF.DAT
            write(io8,903) iyrc,ijulc,ihrc
            write(io8,904) 9999.,9999.,9999,9999,9999.,9999,
     1          9999.,9999
         elseif (opflag.eq.2) then
c ---       HUSWO
c ---       Set date-time for HUSWO (1-24 convention)
            yrh=iyrc
            julh=ijulc
            moh=imoc
            dyh=idyc
            hrh=ihrc
            if(hrh.EQ.0) then
               call INCR(io6,yrh,julh,hrh,-1)
               call GRDAY(io6,yrh,julh,moh,dyh)
               hrh=24
            endif
            write(io8,912) istnid,asosflg,yrh,moh,dyh,hrh,9999,9999,
     &             99,99,999.9,' ',999.9,999,9999,' ',
     &             999,99.9,9999.9,99999,'99999999',
     &             99,999,99,999,99,999,
     &             999,' ',999
         elseif (opflag.eq.3) then
c ---       CD144
c ---       Set date-time for CD144 (YY convention for year)
            icent=iyrc/100
            iyr=iyrc-icent*100
            write(io8,915) istnid,iyr,imoc,idyc,ihrc
         endif

c ---    Increment date-time for next hour
         call INCR(io6,iyrc,ijulc,ihrc,1)
         idathr=iyrc*100000+ijulc*100+ihrc
      endif
      goto 444
      
 999  continue
      
      if(istflag) then
         write(*,*) 'Only last 5 digits of Station ID are kept'
         write(io6,*) 'Only last 5 digits of Station ID are kept'
      endif
      if(nelev.gt.0) then
         write(*,710) nelev,elevation
         write(io6,710) nelev,elevation
 710     format(' ',i5,' station pressures obtained from sea level',
     &        ' Elevation: ',f7.2,' m')
      elseif(nelev.lt.0) then
         write(*,711) -nelev
         write(io6,711) -nelev
 711     format(' ',i5,' station pressures left missing, ',
     &      ' could have been obtained from sea level P')
      endif

      stop
      end

c======================================================================
      subroutine ceilingh(ichcode,ceiling,iyr,iday,ihr)
c======================================================================
c --- This subroutine converts ceiling height code to ceiling height (in m)
c --- All ceiling heights are in meters
      if(ichcode.eq.0) then
        ceiling = 15.
      elseif(ichcode.ge.1.and.ichcode.le.50) then
        ceiling = 30.*ichcode
      elseif(ichcode.ge.51.and.ichcode.le.55) then
        ceiling = 9999.
        write(*,*) iyr,iday,ihr,'illegal ceiling height code:',    
     &       ichcode
      elseif(ichcode.gt.99.or.ichcode.lt.0) then
        ceiling = 9999.
        write(*,*) iyr,iday,ihr,'illegal ceiling height code:',
     &       ichcode
      elseif(ichcode.ge.56.and.ichcode.le.80) then
        ceiling = 1800. + (ichcode - 56)*300.     
      elseif(ichcode.ge.81.and.ichcode.le.88) then
        ceiling = 10500. + (ichcode - 81)*1500.
      elseif(ichcode.eq.89) then
        ceiling = 21000
      elseif(ichcode.eq.90) then
        ceiling = 25
      elseif(ichcode.eq.91) then
        ceiling = 50
      elseif(ichcode.eq.92) then
        ceiling = 100
      elseif(ichcode.eq.93) then
        ceiling = 200
      elseif(ichcode.eq.94) then
        ceiling = 300
      elseif(ichcode.eq.95) then
        ceiling = 600
      elseif(ichcode.eq.96) then
        ceiling = 1000
      elseif(ichcode.eq.97) then
        ceiling = 1500
      elseif(ichcode.eq.98) then
        ceiling = 2000
      elseif(ichcode.eq.99) then
        ceiling = 2500
      endif
      return
      end   
c----------------------------------------------------------------------
       subroutine weather(pwthcd,aww)
c      convert weather codes to CD144 format
c----------------------------------------------------------------------
       character*2 aww(4)
       character*8 pwthcd
       integer iw1(0:99),iw2(0:99),iw1b(0:99),iw2b(0:99)

c data for conversion:
c                0  1  2  3  4  5  6  7  8  9
       data iw1/ 0, 0, 0, 0,31,31,31,30,30,30,
     &          26,30,30, 0, 0, 0, 0,24,24,24,
     &          26,25,27,29,26,25,28,29,30,24,
     &          30,30,30,30,30,30, 0, 0,31,31,
     &          30,30,30,30,30,30,30,30,30,30,
     &          26,26,26,26,26,26,26,26,26,26,
     &          25,25,25,25,25,25,25,25,25,25,
     &          27,27,27,27,27,27,27,28,27,29,
     &          25,25,25,25,25,28,28,29,29,29,
     &          29,24,24,24,24,24,24,24,24,24/
       data iw2/ 0, 0, 0, 0, 3, 2, 4, 4, 4, 4,
     &           4, 3, 3, 0, 0, 0, 0, 1, 5, 3,
     &           5, 2, 2, 2, 8, 5, 2, 5, 1, 1,
     &           4, 4, 4, 4, 4, 4, 0, 0, 5, 5,
     &           1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     &           4, 4, 5, 5, 6, 6, 7, 8, 4, 5,
     &           1, 1, 2, 2, 3, 3, 7, 8, 1, 2,
     &           1, 1, 2, 2, 3, 3, 1, 8, 1, 2,
     &           4, 5, 6, 4, 5, 1, 2, 8, 8, 5,
     &           5, 1, 1, 1, 1, 1, 1, 2, 1, 2/
       data iw1b/ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0, 0, 0, 0, 0, 0,25,25,
     &           0, 0, 0, 0, 0, 0, 0, 0,27,27,
     &           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0,28,28, 0, 0, 0, 0, 0,
     &           0,25,25,27,27,25,29,25,30,29/
       data iw2b/ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0, 0, 0, 0, 0, 0, 1, 2,
     &           0, 0, 0, 0, 0, 0, 0, 0, 1, 2,
     &           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     &           0, 0, 0, 1, 2, 0, 0, 0, 0, 0,
     &           0, 1, 2, 1, 2, 2, 5, 2, 4, 5/

       if (aww(1).eq.'**'.and.aww(2).eq.'**'
     &      .and.aww(3).eq.'**'.and.aww(4).eq.'**') then
c no weather codes: set to missing:
          pwthcd = '        '
       else
c weather codes present: initialise pwthcd and convert codes
          pwthcd = '00000000'
c loop over weather codes (reverse order so that 1 takes precedence over 4
          do i = 4,1,-1
             if (aww(i).ne.'**') then
c read code:
                read (aww(i),*) nw1 
c first mapping:
                ncol = iw1(nw1)-23
                ncode = iw2(nw1)
                if (ncol.ge.0.and.ncode.ne.0) then
                   write (pwthcd(ncol:ncol),'(I1)') ncode
                endif
c second mapping:
                ncol = iw1b(nw1)-23
                ncode = iw2b(nw1)
                if (ncol.ge.0.and.ncode.ne.0) then
                   write (pwthcd(ncol:ncol),'(I1)') ncode
                endif
             endif
          enddo
       endif


       return
       end
c===========================================================
       subroutine pcpcode(aww1,aww2,aww3,ipcode,tempk)
c ---- This subroutine is condensed from Al's previous version. -- SD
c ---- Decode for Manual Station
       Character*3 aww1,aww2,aww3
c       print *, aww1,aww2,aww3
         ipcode = 0
         if(aww1(1:1).eq.'L'.or.aww2(1:1).eq.'L'.or.
     1      aww3(1:1).eq.'L') then
	     ipcode=1
         else if(aww1(1:2).eq.'ZL'.or.aww2(1:2).eq.'ZL'.or.
     1      aww3(1:2).eq.'ZL') then
	     ipcode=20
         else if(aww1(1:1).eq.'R'.or.aww2(1:1).eq.'R'.or.
     1      aww3(1:1).eq.'R') then
	     ipcode=1
         else if(aww1(1:2).eq.'ZR'.or.aww2(1:2).eq.'ZR'.or.
     1      aww3(1:2).eq.'ZR') then
	     ipcode=20
         else if(aww1(1:1).eq.'S'.or.aww2(1:1).eq.'S'.or.
     1      aww3(1:1).eq.'S') then
	     ipcode=20
         else if(aww1(1:2).eq.'IC'.or.aww2(1:2).eq.'IC'.or.
     1      aww3(1:2).eq.'IC') then
	     ipcode=20
         else if(aww1(1:2).eq.'SG'.or.aww2(1:2).eq.'SG'.or.
     1      aww3(1:2).eq.'SG') then
	     ipcode=20
         else if(aww1(1:2).eq.'IP'.or.aww2(1:2).eq.'IP'.or.
     1      aww3(1:2).eq.'IP') then
	     ipcode=20
         else if(aww1(1:1).eq.'A'.or.aww2(1:1).eq.'A'.or.
     1      aww3(1:1).eq.'A') then
	     ipcode=20
         else if(aww1(1:2).eq.'TA'.or.aww2(1:2).eq.'TA'.or.
     1      aww3(1:2).eq.'TA') then
	     ipcode=1
       else if(aww1.eq.'** '.and.aww2.eq.'** '.and.aww3.eq.'** ') then
           ipcode = 0
       endif
c 'use' tempk (eliminates compiler message):
       if (tempk.lt.0) then
          write(*,*) 'Pretty Chilly!'
       endif
c --- Assume precipitation is in the form of frozen p when temperature
c --- is below 273.15K
c      if(ipcode.eq.1.and.tempk.lt.273.15) then
c           ipcode = 20
c      endif
      return
      end

c Subroutine WETBULB------------------------------------------------------
c
c                Written by Lloyd Schulman (adapted from program wetbulb2)
c                Earth Tech
c                September 22, 1999
c
c --- PURPOSE:  Estimates the wet bulb and dew point temperatures. The
c               wet bulb is estimated by stepping through the possible
c               values using a relationship from the Smithsonian
c               Meteorological Tables that approximate the
c               irreversible processes occuring around an actual wet
c               bulb, instead of the reversible isobaric process. The
c               dew point is estimated using Schulman derivation from
c               evaporation curve equation (Clausius-Clapeyron) and
c               RH=es(Td)/es(T).
c
c --- UPDATES:
c ---           1999-09-22 to 2002-03-07, D. Strimaitis
c               Add station pressure (mb) to calling arguments
c               Use TF and TDF as inputs (they are in DATSAV file)
c               Replace saturation pressure function
c               
c
c --- INPUTS:
c               TF - real              - dry bulb temperature (deg F)
c              TDF - real              - Dewpoint temperature (deg F)
c             PSTA - real              - station pressure (mb)
c
c --- OUTPUT:
c              TWF - real              - Wet bulb temperature (deg F)
c               RH - real              - Relative humidity (fraction)
c
c----------------------------------------------------------------------
      subroutine wetbulb (tf,tdf,psta,twf,rh)

c Convert to deg C
      TC = (TF-32.0)/1.8
      TDC = (TDF-32.0)/1.8
c Convert to deg K
      TK = TC + 273.16
      TDK = TDC + 273.16

c Calculate saturation vapor pressure for T (K)
      ES = vsat(TK)
c Calculate saturation vapor pressure for TD (K)
      E = vsat(TDK)
c Calculate relative humidity
      RH = E/ES

c Estimate wet bulb using Smithsonian equation
c Temperature in deg C; pressure in mb
c TW must be between TD and T (all equal if RH=1.00)
c Increment from TD to T in 0.05 deg C steps

c Set pressure term for temperature in degrees C
      ptermC=1515./psta

      n=1
100   TWetK = TDK + (n-1)*0.05
c check to see that TWetK is in range
      if(TWetK .GT. TK)go to 150
      EW = vsat(TWetK)
      TWC = TC - ptermC*(EW-E)
      TWK = TWC + 273.16
c see if solution to wet bulb equation (cross-over point twet, tw)
      if (twetK.ge.(twK-0.025)) then
         twK = 0.5*(twetK+twK)
         goto 200
      else
         n = n+1
         goto 100
      endif
150   write(*,*) 'Error Calculating TwK:',twK,'TwetK:',twetK,
     &     ' TK:',TK,' RH:',rh,' TDK:',TDK
      TWF = 999.9

      return

c     convert to deg F
200   continue
      TWF = 1.8*(TWK-273.15) + 32.0

c      write(*,*) TWF,TDF,TF,RH

      return
      end


c----------------------------------------------------------------------
      function vsat(tdegk)
c----------------------------------------------------------------------
c
c --- DATSAV     Version: 1.0      Level: 020307                  VSAT
c                J. Scire, Earth Tech
c
c --- PURPOSE:  Compute saturation vapor pressure for given temperature
c
c --- INPUTS:
c          tdegk - real       - Temperature (K)
c
c --- OUTPUT:
c           vsat - real       - Saturation water vapor pressure (mb)
c
c --- VSAT called by:   
c --- VSAT calls:       none
c----------------------------------------------------------------------
c
      data gascon/8.31372e-2/

c --- Compute saturation vapor pressure (Goff-Gratch eqns.)
      trat=273.16/tdegk
      if(tdegk.lt.273.16)then
c ---    temperature below 0. deg. C
         vsat=exp(2.303*(-9.09685*(trat-1.0)+
     1   0.87682*(1.0-tdegk/273.16))-3.56654*(5.6100577-alog(tdegk))+
     2   1.81)
      else
c ---    temperature above 0. deg. C
         vsat=exp(2.303*(10.79574*(1.0-trat)+
     1   1.50475e-4*(1.0-10.**(-8.2969*(tdegk/273.16-1.0)))+
     2   4.2873e-4*(10.**(4.76955*(1.0-trat))-1.0))-
     3   5.028*(alog(tdegk)-5.6100577)+1.81)
      endif

      return
      end



c----------------------------------------------------------------------
      subroutine yr4(io,iyr,ierr)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 1.0      Level: 991104                    YR4
c ---            D. Strimaitis,   Earth Tech
c
c --- PURPOSE: Checks/converts 2-digit year to 4-digit year
c
c --- INPUTS:
c                IO - integer    - Unit number for list file output
c               IYR - integer    - Year (YYYY or YY)
c
c     Common block /Y2K/:
c             IYYLO - integer    - Smallest 2-digit year for which
c                                  'old' century marker is used
c             ICCLO - integer    - 2-digit ('old') century
c
c --- OUTPUT:
c               IYR - integer    - Year (YYYY)
c              IERR - integer    - Error code: 0=OK, 1=FATAL
c
c --- YR4 called by:  Input routines reading 'year' data
c --- YR4 calls:      none
c----------------------------------------------------------------------
c
      common/y2k/iyylo,icclo

      ierr=0

c --- Test for 4-digit year (must exceed 1000)
      if(iyr.GT.1000) then
c ---    Passes 11th Century test (large year not trapped)
         return
      elseif(iyr.LT.100 .AND. iyr.GE.0) then
c ---    2-digit year
c ---    Construct 4-digit year
         if(iyr.LT.iyylo) then
            iyr=(icclo+1)*100+iyr
         else
            iyr=icclo*100+iyr
         endif
      else
c ---    Year not recognized
         ierr=1
         write(io,*)'ERROR in YR4 --- Year not recognized: ',iyr
         write(*,*)'ERROR in YR4 --- Year not recognized: ',iyr
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine yr4c(iyr)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 1.0      Level: 991104                   YR4C
c ---            D. Strimaitis,  Earth Tech
c
c --- PURPOSE: Checks/converts 2-digit year to 4-digit year (CURRENT)
c
c --- INPUTS:
c               IYR - integer    - Year (YYYY or YY)
c
c --- OUTPUT:
c               IYR - integer    - Year (YYYY)
c
c --- YR4C called by:  host subroutines
c --- YR4C calls:      none
c----------------------------------------------------------------------
c --- Set parameters for converting a current year (1999 - 2098)
c --- Use KCCLO as century digits for years GE KYYLO
      data kyylo/99/, kcclo/19/

c --- Construct 4-digit year
      if(iyr.LT.kyylo) then
         iyr=(kcclo+1)*100+iyr
      else
         iyr=kcclo*100+iyr
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine qayr4(io,iyr,metrun,ierr)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 1.0      Level: 991104                  QAYR4
c ---            D. Strimaitis,   Earth Tech
c
c --- PURPOSE: Defines century and year markers to use in converting
c ---          2-digit year to 4-digit year
c ---          The IBYR (YYYY) must be provided in the control file
c
c --- INPUTS:
c                IO - integer    - Unit number for list file output
c               IYR - integer    - Year provided for start of run
c            METRUN - integer    - Flag to run period in met file
c                                  0 = do not run period
c                                  1 = run period
c
c --- OUTPUT:
c              IERR - integer    - Error code: 0=OK, 1=FATAL
c
c     Common block /Y2K/:
c             IYYLO - integer    - Smallest 2-digit year for which
c                                  'old' century marker is used
c             ICCLO - integer    - 2-digit ('old') century
c
c --- QAYR4 called by:  host subroutines
c --- QAYR4 calls:      none
c----------------------------------------------------------------------
c
      common/y2k/iyylo,icclo

c --- Sets parameters for the starting century marker (CC) and the
c --- 2-digit year (YY) used as the marker between the starting century
c --- and the next century.  For example, if CC=19 and YY=30, then a
c --- year less than 30 (say 15) is assumed to be 2015.  Any year
c --- greater than or equal to 30 (say 56) is assumed to be 1956.

c --- Set number of years prior to start of simulation that must not
c --- be placed in the next century
      data ibackyr/50/

      ierr=0

c --- Expect explicit starting year (YYYY)
c --- Test for 4-digit year (must exceed 1000)
      if(iyr.GT.1000) then
c ---    Passes 11th Century test (large year not trapped)
c ---    Back up IBACKYR years to set IYYLO
         kyr=iyr-ibackyr
c ---    Extract starting 2-digit century and 2-digit year
         icclo=kyr/100
         iyylo=kyr-icclo*100

c ---    Warn user that control file input is used to convert to YYYY
         iyr1=icclo*100+iyylo
         iyr2=(icclo+1)*100+iyylo-1
         write(io,*)
         write(io,*)'-------------------------------------------------'
         write(io,*)'NOTICE: Starting year in control file sets the'
         write(io,*)'        expected century for the simulation.  All'
         write(io,*)'        YY years are converted to YYYY years in'
         write(io,*)'        the range: ',iyr1,iyr2
         write(io,*)'-------------------------------------------------'
         write(io,*)
      else
         ierr=1
         write(*,*)
         write(*,*)'--------------------------------------------'
         write(*,*)'QAYR4 -- Start year must be 4-digits!: ',iyr
         if(metrun.EQ.1) then
            write(*,*)'         and must always be provided'
         endif
         write(*,*)'--------------------------------------------'
         write(*,*)
         write(io,*)
         write(io,*)'-------------------------------------------'
         write(io,*)'QAYR4 -- Start year must be 4-digits!: ',iyr
         if(metrun.EQ.1) then
            write(io,*)'         and must always be provided'
         endif
         write(io,*)'-------------------------------------------'
         write(io,*)
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine julday(io,iyr,imo,iday,ijuldy)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 1.0      Level: 000602                 JULDAY
c ---            J. Scire, SRC
c
c --- PURPOSE:  Compute the Julian day number from the Gregorian
c               date (month, day)
c
c --- UPDATE
c ---               000602  (DGS): YYYY format for year
c
c --- INPUTS:
c            IO - integer      - Unit number for list file output
c           IYR - integer      - Year
c           IMO - integer      - Month
c          IDAY - integer      - Day
c
c --- OUTPUT:
c          IJUL - integer      - Julian day
c
c --- JULDAY called by:  host subroutines
c --- JULDAY calls:      none
c----------------------------------------------------------------------
c
      integer kday(12)
      data kday/0,31,59,90,120,151,181,212,243,273,304,334/
c
c --- Check for valid input data
      ierr=0
c --- Check for valid month
      if(imo.lt.1.or.imo.gt.12)ierr=1
c --- Check for valid day in 30-day months
      if(imo.eq.4.or.imo.eq.6.or.imo.eq.9.or.imo.eq.11)then
         if(iday.gt.30)ierr=1
      else if(imo.eq.2)then
         if(mod(iyr,4).eq.0)then
c ---       February in a leap year
            if(iday.gt.29)ierr=1
         else
c ---       February in a non-leap year
            if(iday.gt.28)ierr=1
         endif
      else
c ---    Check for valid day in 31-day months
         if(iday.gt.31)ierr=1
      endif
c
      if(ierr.eq.1)then
         write(io,*)
         write(io,*)'ERROR in SUBR. JULDAY'
         write(io,*)'Invalid date - IYR = ',iyr,' IMO = ',
     1    imo,' IDAY = ',iday
         write(*,*)
         stop 'Halted in JULDAY -- see list file.'
      endif
c
c --- Compute the Julian day
      ijuldy=kday(imo)+iday
      if(imo.le.2)return
      if(mod(iyr,4).EQ.0)ijuldy=ijuldy+1
c
      return
      end
c----------------------------------------------------------------------
      subroutine grday(io,iyr,ijul,imo,iday)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 1.0      Level: 000602                  GRDAY
c                J. Scire, SRC
c
c --- PURPOSE:  Compute the Gregorian date (month, day) from the
c               Julian day
c
c --- UPDATE
c ---               000602  (DGS): YYYY format for year
c
c --- INPUTS:
c            IO - integer      - Unit number for list file output
c           IYR - integer      - Year
c          IJUL - integer      - Julian day
c
c --- OUTPUT:
c           IMO - integer      - Month
c          IDAY - integer      - Day
c
c --- GRDAY called by:  host subroutines
c --- GRDAY calls:      none
c----------------------------------------------------------------------
c
      integer kday(12,2)
      data kday/31,59,90,120,151,181,212,243,273,304,334,365,
     1          31,60,91,121,152,182,213,244,274,305,335,366/
c
c
      ileap=1
      if(mod(iyr,4).eq.0)ileap=2
      if(ijul.lt.1.or.ijul.gt.kday(12,ileap))go to 11
c
      do 10 i=1,12
      if(ijul.gt.kday(i,ileap))go to 10
      imo=i
      iday=ijul
      if(imo.ne.1)iday=ijul-kday(imo-1,ileap)
      return
10    continue
c
11    continue
      write(io,12)iyr,ijul
12    format(//2x,'ERROR in SUBR. GRDAY -- invalid Julian day '//2x,
     1 'iyr = ',i5,3x,'ijul = ',i5)
      write(*,*)
      stop 'Halted in GRDAY -- see list file.'
      end
c------------------------------------------------------------------------------
      subroutine dedat(idathr,iyr,ijul,ihr)
c------------------------------------------------------------------------------
c
c --- CALUTILS   Version: 1.0      Level: 941215                  DEDAT
c ---            J. Scire, SRC
c
c --- Decode a date-time variable
c
c --- INPUTS:
c            IDATHR - integer    - Date-time variable (YYYYJJJHH)
c
c --- OUTPUT:
c               IYR - integer    - Year of precip. data (4 digits)
c              IJUL - integer    - Julian day number of precip. data
c               IHR - integer    - Ending hour (1-24) of precip. data
c
c --- DEDAT called by:  host subroutines
c --- DEDAT calls:      none
c------------------------------------------------------------------------------
c
c --- decode date and time
      iyr=idathr/100000
      ijul=idathr/100-iyr*1000
      ihr=idathr-iyr*100000-ijul*100
c
      return
      end
c------------------------------------------------------------------------------
      subroutine deltt(j1yr,j1jul,j1hr,j2yr,j2jul,j2hr,jleng)
c------------------------------------------------------------------------------
c
c --- CALUTILS   Version: 1.0      Level: 941215                  DELTT
c ---            J. Scire, SRC
c
c --- Compute the difference (in hours) between two dates & times
c ---    (time #2 - time #1)
c
c --- INPUTS:
c              J1YR - integer    - Year of date/time #1
c             J1JUL - integer    - Julian day of date/time #1
c              J1HR - integer    - Hour of date/time #1
c              J2YR - integer    - Year of date/time #2
c             J2JUL - integer    - Julian day of date/time #2
c              J2HR - integer    - Hour of date/time #2
c
c --- OUTPUT:
c             JLENG - integer    - Difference (#2 - #1) in hours
c
c --- DELTT called by:  host subroutines
c --- DELTT calls:      none
c------------------------------------------------------------------------------
c
      jmin=min0(j1yr,j2yr)
c
c --- find the number of hours between Jan. 1 of the "base" year and
c --- the first date/hour
      if(j1yr.eq.jmin)then
         j1=0
      else
         j1=0
         j1yrm1=j1yr-1
         do 10 i=jmin,j1yrm1
         if(mod(i,4).eq.0)then
            j1=j1+8784
         else
            j1=j1+8760
         endif
10       continue
      endif
      j1=j1+(j1jul-1)*24+j1hr
c
c --- find the number of hours between Jan. 1 of the "base" year and
c --- the second date/hour
      if(j2yr.eq.jmin)then
         j2=0
      else
         j2=0
         j2yrm1=j2yr-1
         do 20 i=jmin,j2yrm1
         if(mod(i,4).eq.0)then
            j2=j2+8784
         else
            j2=j2+8760
         endif
20       continue
      endif
      j2=j2+(j2jul-1)*24+j2hr
c
c --- compute the time difference (in hours)
      jleng=j2-j1
c
      return
      end
c----------------------------------------------------------------------
      subroutine incr(io,iyr,ijul,ihr,nhrinc)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 1.0      Level: 000602                   INCR
c                J. Scire, SRC
c
c --- PURPOSE:  Increment the time and date by "NHRINC" hours
c
c --- UPDATE
c ---               000602  (DGS): add message to "stop"
c ---               980304  (DGS): Allow for a negative "increment" of
c                                  up to 24 hours
c ---               980304  (DGS): Allow for arbitrarily large nhrinc
c
c --- INPUTS:
c       IO     - integer - Unit number for list file output
c       IYR    - integer - Current year
c       IJUL   - integer - Current Julian day
c       IHR    - integer - Current hour (00-23)
c       NHRINC - integer - Time increment (hours)
c
c               NOTE: "NHRINC" must >= -24
c                      Hour is between 00-23
c
c --- OUTPUT:
c       IYR    - integer - Updated year
c       IJUL   - integer - Updated Julian day
c       IHR    - integer - Updated hour (00-23)
c
c --- INCR called by: host subroutines
c --- INCR calls:     none
c----------------------------------------------------------------------
c
c --- Check nhrinc
      if(nhrinc.lt.-24) then
         write(io,*)'ERROR IN SUBR. INCR -- Invalid value of NHRINC ',
     1   '-- NHRINC = ',nhrinc
         write(*,*)
         stop 'Halted in INCR -- see list file.'
      endif

c --- Save increment remaining (needed if nhrinc > 8760)
      nleft=nhrinc
c
c --- Process change in hour
      if(nhrinc.gt.0)then
c
10       ninc=MIN0(nleft,8760)
         nleft=nleft-ninc
c
c ---    Increment time
         ihr=ihr+ninc
         if(ihr.le.23)return
c
c ---    Increment day
         ijul=ijul+ihr/24
         ihr=mod(ihr,24)
c
c ---    ILEAP = 0 (non-leap year) or 1 (leap year)
         if(mod(iyr,4).eq.0)then
            ileap=1
         else
            ileap=0
         endif
c
         if(ijul.gt.365+ileap) then
c ---       Update year
            iyr=iyr+1
            ijul=ijul-(365+ileap)
         endif
c
c ---    Repeat if more hours need to be added
         if(nleft.GT.0) goto 10
c
      elseif(nhrinc.lt.0)then
c ---    Decrement time
         ihr=ihr+nhrinc
         if(ihr.lt.0)then
            ihr=ihr+24
            ijul=ijul-1
            if(ijul.lt.1)then
               iyr=iyr-1
               if(mod(iyr,4).eq.0)then
                  ijul=366
               else
                  ijul=365
               endif
            endif
         endif
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine comline(ctext)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.52     Level: 040330                COMLINE
c                J. Scire, SRC
c
c --- PURPOSE:  Call the compiler-specific system routine that will
c               pass back the command line argument after the text
c               that executed the program
c
c               This routine contains calls for several different
c               compilers, but only one should be active at any one
c               time.
c
c --- UPDATE
c --- V2.2 (960521) to V2.3 (040330)  DGS
c               - Replace strings c*70 with c*132
c
c --- INPUTS:
c
c          CTEXT - character*132 - Default command line argument #1
c
c --- OUTPUT:
c
c          CTEXT - character*132 - Command line argument #1
c                                  If command line argument is
c                                  missing, CTEXT is not changed
c
c --- COMLINE called by:  SETUP
c --- COMLINE calls:      GETCL - Lahey system routine
c                         NARGS, GETARG - Microsoft routines
c                         NARGS, GETARG - Compaq DF routines
c                         IARGC, GETARG - Sun routines
c                         IARGC, GETARG - HP routines
c----------------------------------------------------------------------
c
c ----------------------------------------
c --- COMPAQ DF compiler library directive
c *** USE DFLIB
c ----------------------------------------
c
      character*132 ctext,cdeflt
c
c --- Microsoft variables
c *** integer*2 iarg,istat
c
c --- HP declaration
c *** external getarg
c *** external iargc
c
c --- The following is for any system without a command line routine
c --- and is also used as a default
      cdeflt=ctext
c
c ------------------
c --- Lahey compiler
c ------------------
      call getcl(ctext)
c
c ----------------------
c --- COMPAQ DF compiler
c ----------------------
c ***  numargs=NARGS()
c ***  if(numargs.ge.1)then
c ***     call GETARG(1,ctext)
c ***  endif
c
c ----------------------
c --- Microsoft compiler
c ----------------------
c *** numargs=NARGS()
c *** if(numargs.ge.1)then
c ***    iarg=1
c ***    call GETARG(iarg,ctext,istat)
c *** endif
c
c ----------------
c --- Sun compiler
c ----------------
c *** numargs=IARGC()
c *** if(numargs.ge.1)then
c ***    call GETARG(1,ctext)
c *** endif
c
c -----------------------------------------------------
c --- HP compiler -- NOTE: needs +U77 switch on compile
c -----------------------------------------------------
c *** numargs=IARGC()
c *** if(numargs.ge.1)then
c ***    call GETARG(1,ctext)
c *** endif
c
c
c --- If no command line arguments, use default
      if(ctext(1:1).eq.' ')ctext=cdeflt

      return
      end
